/********************************************************************
     Filename: piccode.c
     Created:  Tuesday, June 24, 2008
     Author(s):
     History:

     Purpose:

     Copyrights(c) 2008
*********************************************************************/

#include <system.h>
#include "logger.h"
#include "piccode.h"

short ccpr1_clone@0x15;
short ccpr2_clone@0x16;
//char i8_I2CData[I2C_BUFFER_SIZE];

/********************************************************************
     Name:          i2c_stop
     Purpose:  Sends a stop signal to the I2C bus.
     Parameters:    None.
     Returns:  None.
*********************************************************************/
void i2c_stop( void )
{
     // don't assume SCL is high on entry
     I2C_SDA_LOW();                // ensure data is low first
     I2C_SCL_HIGH();
     I2C_DELAY_1uS();
     I2C_SCL_DIR_INPUT();          // float clock high
     I2C_DELAY_4uS();
     I2C_SDA_HIGH();                    // the low->high data transition
     I2C_DELAY_5uS();              // bus free time before next start
     I2C_SDA_DIR_INPUT();          // float data high
}

/********************************************************************
     Name:          i2c_wait_for_scl
     Purpose:  Waits for the clock to be released by the slave.
     Parameters:    None.
     Returns:  TRUE if failed, FALSE otherwise.
*********************************************************************/
char i2c_wait_for_scl( void )
{
     if ( !I2C_SCL )               // SCL_DIR should be input here
     {
          I2C_DELAY_10uS();
          if ( !I2C_SCL )          // if the clock is still low -> bus error
          {
               return I2C_ERROR;
          }
     }
     return 0;
}

/********************************************************************
     Name:          i2c_read_acknowledge
     Purpose:  Reads acknowledge signal from slave device.
     Parameters:    None.
     Returns:  0xFF if bus error. If slave acknowledges ack = 0.
                    ack = 1 otherwise.
*********************************************************************/
char i2c_read_acknowledge( void )
{
     char ack;

     I2C_SCL_LOW();                // make clock low
     I2C_SDA_DIR_INPUT();          // disable data line - listen for ack
     I2C_DELAY_4uS();              // SCL low to data out valid
     I2C_SCL_DIR_INPUT();          // float clock high
     I2C_DELAY_1uS();
     ack = I2C_SDA;
     // wait for slave to release clock line after processing byte
//   if ( i2c_wait_for_scl() )
//        return I2C_ERROR;
//patched
     I2C_SCL_LOW();                // make clock low
//patched

     return ack;
}

/********************************************************************
     Name:          i2c_send_acknowledge
     Purpose:  Sends acknowledge signal.
     Parameters:    more, a value of zero means last byte was sent.
*********************************************************************/

void i2c_send_acknowledge( char more )
{
     I2C_SCL_LOW();
     if ( more )
     {
          I2C_SDA_LOW();           // drive line low - more to come
     }
     else
     {
          I2C_SDA_HIGH();
     }
     I2C_DELAY_1uS();
     I2C_SCL_DIR_INPUT();          // float clock high
     I2C_DELAY_4uS();
}

/********************************************************************
     Name:          i2c_send_byte
     Purpose:  Sends a byte through the I2C bus.
     Parameters:    byte is the byte to output.
     Returns:  Returns the output of i2c_read_acknowledge().
*********************************************************************/
char i2c_send_byte( char byte )
{
     char i;

     for ( i=0;i<8;i++ )
     {
          I2C_SCL_LOW();                     // drive clock low
          // data hold time = 0, send data now
          if ( byte&0x80 != 0 )
          {
               I2C_SDA_HIGH();
          }
          else
          {
               I2C_SDA_LOW();
          }
          I2C_DELAY_1uS();
          byte = byte << 1;
          I2C_SCL_DIR_INPUT();               // float clock high
//        if ( i2c_wait_for_scl() )
//             return I2C_ERROR;
//patched
I2C_SCL_LOW();                     // drive clock low
//patched

          I2C_DELAY_4uS();
     }
     return ( i2c_read_acknowledge() ); // returns ack, ~ack
}

/********************************************************************
     Name:          i2c_read_byte
     Purpose:  Reads a byte on the I2C bus.
     Parameters:    None.
     Returns:  Returns the byte read.
*********************************************************************/
char i2c_read_byte( void )
{
     char i;
     char byte = 0;

     for ( i=0;i<8;i++ )
     {
          I2C_SCL_LOW();                // drive clock low
          I2C_DELAY_5uS();              // min clock period
          I2C_SDA_DIR_INPUT();          // release data line
          I2C_SCL_DIR_INPUT();          // float clock high
          if ( i2c_wait_for_scl() )
               return I2C_ERROR;
          I2C_DELAY_4uS();
          byte = byte << 1;             // read the next bit
          if (I2C_SDA) byte |= 1;
     }
     return byte;
}

/********************************************************************
     Name:          i2c_open
     Purpose:  Opens communication with a device sending
                    the control byte.
     Returns:  I2C_ERROR if address is not acknowledged
*********************************************************************/
char i2c_open( char control )
{
     // start sequence
     I2C_SCL_LOW();                     // ensure clock is low
     I2C_SDA_HIGH();                         // ensure data is high
     I2C_DELAY_1uS();
     I2C_SCL_DIR_INPUT();               // clock pulse high
     I2C_DELAY_4uS();
     I2C_SDA_LOW();                     // the high->low transition
     I2C_DELAY_4uS();
     // send the control byte to slave device
     return ( i2c_send_byte(control) );
}


/** \brief Loads TXREG with data.

  This function can be used to send data using the PIC's AUSART.
  \param char to be loaded into TXREG.
*/
void txreg_putch( char c )
{
     clear_wdt();
     set_bit( pie1, TXIE );
     while( !(txsta&0x02) );
     txreg = c;
}
